import pygame
import random
import math
import serial
import serial.tools.list_ports

pygame.init()
clock = pygame.time.Clock()
SCREEN_WIDTH = 1400
SCREEN_HEIGHT = 900
BACKGROUND_COLOR = (0, 0, 0)
screen = pygame.display.set_mode((SCREEN_WIDTH, SCREEN_HEIGHT))
pygame.display.set_caption("Zuma Game")

RedLineLen=200


MyCOM = serial.Serial()#串口
MyCOM.port="COM7"
MyCOM.baudrate = 115200
MyCOM.bytesize = 8
MyCOM.parity = 'N'
MyCOM.stopbits = 1
try:
    MyCOM.open()
except:
    print("Open Com Err")

class Bullet:
    def __init__(self, x, y, angle, color):
        self.x = x
        self.y = y
        self.angle = angle
        self.speed = 20
        self.color = color
        self.radius = 20

    def move(self):
        self.x += self.speed * math.cos(self.angle)
        self.y += self.speed * math.sin(self.angle)

    def draw(self):
        pygame.draw.circle(screen, self.color, (int(self.x), int(self.y)), self.radius)

class Cannon:
    def __init__(self):
        self.x = SCREEN_WIDTH / 2       #800
        self.y = SCREEN_HEIGHT-100  #600
        self.angle = 0
        self.color = (0, 0, 255)
        self.width = 20
        self.height = 20
        self.mouse_pos = (self.x, self.y)

    def aim(self):
        self.mouse_pos = pygame.mouse.get_pos()
        self.angle = math.atan2(self.mouse_pos[1] - self.y, self.mouse_pos[0] - self.x)

    def draw(self):
        pygame.draw.rect(screen, self.color, (self.x - self.width / 2, self.y - self.height / 2, self.width, self.height))
        pygame.draw.line(screen, self.color, (self.x, self.y), (self.x + self.width / 2 * math.cos(self.angle), self.y + self.width / 2 * math.sin(self.angle)), 5)
class Ball:
    def __init__(self):
        self.x = random.randint(0, SCREEN_WIDTH)
        self.y = 0
        self.speed = random.random()*0.5+0.5
        self.color = (random.randint(0, 255), random.randint(0, 255), random.randint(0, 255))
        self.radius = random.randint(20, 80)
        self.broken = False

    def update(self):
        self.y += self.speed
        if self.y > SCREEN_HEIGHT:
            self.broken = True
    def draw(self):
        pygame.draw.circle(screen, self.color, (self.x, self.y), self.radius)

cannon = Cannon()
bullets = []
balls = []
continuous_fire = False
Continue_Div=0
LastAngle=0.0
LastShootIndex=0
CurrentShootIndex=0
CurrentBallColorR=0
CurrentBallColorG=0
CurrentBallColorB=0

while True:
    angle=-999.0
    screen.fill(BACKGROUND_COLOR)
    if MyCOM.is_open:
        UartRxLen = MyCOM.inWaiting()
        # print("Uart RX:"+str(myUart.ser.inWaiting()))
        if (UartRxLen > 4):
            UartRxData = MyCOM.read(UartRxLen)#5
            # MyCOM.readall()
            # angle=(3480-int(UartRxData))/(3480-450)*180
            angle=(3480-int(UartRxData[UartRxLen-6]*256+UartRxData[UartRxLen-5]))/(3480-450)*180
            CurrentShootIndex=UartRxData[UartRxLen-7]
            if LastShootIndex != CurrentShootIndex:
                CurrentBallColorR = UartRxData[UartRxLen-4]
                CurrentBallColorG = UartRxData[UartRxLen-3]
                CurrentBallColorB = UartRxData[UartRxLen-2]
            # try:
            #     # print((UartRxData).hex())
            #     print(int(angle))      #450-3480  180度到0度范围
            # except Exception as e:
            #     print(e)

    if angle!=-999: #用这个判断有没有读取到串口数据更新角度，不然异常
        LastAngle=angle
    pygame.draw.line(screen, (240, 0, 0), (cannon.x, cannon.y), (cannon.x-RedLineLen*math.cos(LastAngle*3.14/180),cannon.y-RedLineLen*math.sin(LastAngle*3.14/180)),5)

    if LastShootIndex != CurrentShootIndex:     #单片机的按键控制发射子弹，这个index是不断自增
        cannon.color = (CurrentBallColorR*250, CurrentBallColorG*250, CurrentBallColorB*250)
        bullet = Bullet(cannon.x, cannon.y, (LastAngle + 180) * 3.14 / 180, cannon.color)
        bullets.append(bullet)
        LastShootIndex = CurrentShootIndex


    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            pygame.quit()
            # sys.exit()
        elif event.type == pygame.MOUSEBUTTONDOWN and event.button == 1:
            # bullet = Bullet(cannon.x, cannon.y, cannon.angle, cannon.color)
            bullet = Bullet(cannon.x, cannon.y, (LastAngle+180) * 3.14 / 180, cannon.color)
            bullets.append(bullet)
            cannon.color = (random.randint(0, 255), random.randint(0, 255), random.randint(0, 255))
        elif event.type == pygame.MOUSEBUTTONDOWN and event.button == 3:
            continuous_fire = True
        elif event.type == pygame.MOUSEBUTTONUP and event.button == 3:
            continuous_fire = False

    cannon.aim()
    cannon.draw()

    if continuous_fire:
        Continue_Div+=1
        if Continue_Div>2:
            Continue_Div=0
            # bullet = Bullet(cannon.x, cannon.y, cannon.angle, cannon.color)
            bullet = Bullet(cannon.x, cannon.y, (LastAngle+180) * 3.14 / 180, cannon.color)
            bullets.append(bullet)
            cannon.color = (random.randint(0, 255), random.randint(0, 255), random.randint(0, 255))

    for bullet in bullets:
        bullet.move()
        # if bullet.x < 1 or bullet.x > SCREEN_WIDTH or bullet.y < 0 or bullet.y > SCREEN_HEIGHT:
        if bullet.x < 1 or bullet.x > SCREEN_WIDTH or bullet.y < 0 :
            bullets.remove(bullet)
            continue        #这里必须有continue，否则Remove后下面继续执行draw也能正常执行，就是列表虽然删除，也可以寻址拿出来，居然没有报错【大坑大坑】
        # if bullet.x < 1:           #【Bug??】 符合条件也没有remove，看是哪里问题
        #     print(bullet.x)
        #     bullets.remove(bullet)
        # if(bullet.x>0):
        #     bullet.draw()
        bullet.draw()

        for ball in balls:
            if ball.y>SCREEN_HEIGHT:
                balls.remove(ball)
            rect = pygame.Rect(ball.x-ball.radius, ball.y-ball.radius, ball.radius*2, ball.radius*2)
            if rect.collidepoint(bullet.x,bullet.y):        #collidecircle  colliderect
                # print("Hit !")
                bullets.remove(bullet)
                ball.radius-=5
                if ball.radius<15:
                    balls.remove(ball)
                break

    # 更新小球状态
    for ball in balls:
        ball.update()

    # 生成新的小球
    if len(balls)<3:        #控制球的数量
        if random.randint(0, 200) < 10:
            balls.append(Ball())

    for ball in balls:
        ball.draw()

    # 设置字体
    font = pygame.font.Font(None, 50)
    score_text = font.render("Angle: {}".format(round(LastAngle,1)), True, (100, 200, 0))
    screen.blit(score_text, (10, 10))

    clock.tick(60)
    pygame.display.update()